@@ -1,6 +1,6 @@
 package Bitcoin::Crypto;
-$Bitcoin::Crypto::VERSION = '1.006_02'; # TRIAL
-$Bitcoin::Crypto::VERSION = '1.00602';
+$Bitcoin::Crypto::VERSION = '1.008_01'; # TRIAL
+$Bitcoin::Crypto::VERSION = '1.00801';
 use v5.10;
 use strict;
 use warnings;
@@ -12,34 +12,43 @@ our %EXPORT_TAGS = (all => [@EXPORT_OK]);
 sub btc_extprv
 {
 	require Bitcoin::Crypto::Key::ExtPrivate;
-	return "Bitcoin::Crypto::Key::ExtPrivate";
+	return 'Bitcoin::Crypto::Key::ExtPrivate';
 }
 
 sub btc_prv
 {
 	require Bitcoin::Crypto::Key::Private;
-	return "Bitcoin::Crypto::Key::Private";
+	return 'Bitcoin::Crypto::Key::Private';
 }
 
 sub btc_extpub
 {
 	require Bitcoin::Crypto::Key::ExtPublic;
-	return "Bitcoin::Crypto::Key::ExtPublic";
+	return 'Bitcoin::Crypto::Key::ExtPublic';
 }
 
 sub btc_pub
 {
 	require Bitcoin::Crypto::Key::Public;
-	return "Bitcoin::Crypto::Key::Public";
+	return 'Bitcoin::Crypto::Key::Public';
 }
 
 sub btc_script
 {
 	require Bitcoin::Crypto::Script;
-	return "Bitcoin::Crypto::Script";
+	return 'Bitcoin::Crypto::Script';
 }
 
+# OO interface
+
+sub extprv { goto \&btc_extprv }
+sub prv { goto \&btc_prv }
+sub extpub { goto \&btc_extpub }
+sub pub { goto \&btc_pub }
+sub script { goto \&btc_script }
+
 __END__
+
 =head1 NAME
 
 Bitcoin::Crypto - Bitcoin cryptography in Perl
@@ -59,16 +68,16 @@ Bitcoin::Crypto - Bitcoin cryptography in Perl
 	my $priv = $derived_key->get_basic_key();
 	my $pub = $priv->get_public_key();
 
-	say "private key: " . $priv->to_wif();
-	say "public key: " . $pub->to_hex();
-	say "address: " . $pub->get_segwit_address();
+	say 'private key: ' . $priv->to_wif();
+	say 'public key: ' . $pub->to_hex();
+	say 'address: ' . $pub->get_segwit_address();
 
-	my $message = "Hello CPAN";
+	my $message = 'Hello CPAN';
 	my $signature = $priv->sign_message($message);
 
 	if ($pub->verify_message($message, $signature)) {
 		say "successfully signed message '$message'";
-		say "signature: " . unpack "H*", $signature;
+		say 'signature: ' . unpack 'H*', $signature;
 	}
 
 =head1 DESCRIPTION
@@ -149,6 +158,8 @@ Most packages in this module have the types of their thrown exceptions documente
 
 =head1 SHORTCUT FUNCTIONS
 
+=head2 Exported interface
+
 This package exports the following functions when asked for them. They are shourtcut functions and will load needed packages and return their names. You can then use names of loaded packages to instantiate them however you want. You can also load all of them with the I<:all> tag in import. These functions can be used as follows:
 
 	use Bitcoin::Crypto qw(btc_pub);
@@ -157,26 +168,36 @@ This package exports the following functions when asked for them. They are shour
 	# we can now use it to run its methods
 	my $public_key = btc_pub->from_hex($hex_data);
 
-=head2 btc_extprv
+=head3 btc_extprv
 
 Loads L<Bitcoin::Crypto::Key::ExtPrivate>
 
-=head2 btc_prv
+=head3 btc_prv
 
 Loads L<Bitcoin::Crypto::Key::Private>
 
-=head2 btc_extpub
+=head3 btc_extpub
 
 Loads L<Bitcoin::Crypto::Key::ExtPublic>
 
-=head2 btc_pub
+=head3 btc_pub
 
 Loads L<Bitcoin::Crypto::Key::Public>
 
-=head2 btc_script
+=head3 btc_script
 
 Loads L<Bitcoin::Crypto::Script>
 
+=head2 OO interface
+
+Alternatively, all function names mentioned above can be used without the C<btc_> prefix in OO style on the package's name.
+
+	use Bitcoin::Crypto;
+
+	# loads Bitcoin::Crypto::Key::Public and returns package name
+	# we can now use it to run its methods
+	my $public_key = Bitcoin::Crypto->pub->from_hex($hex_data);
+
 =head1 DISCLAIMER
 
 Although the module was written with an extra care and appropriate tests are in place asserting compatibility with many Bitcoin standards, due to complexity of the subject some bugs may still be present. In the world of digital money, a single bug may lead to losing funds. I encourage anyone to test the module themselves, review the test cases and use the module with care. Suggestions for improvements and more edge cases to test will be gladly accepted, but there is no warranty on your funds being manipulated by this module.
@@ -188,10 +209,16 @@ The module have a little bit of startup time because of Moo and Type::Tiny, meas
 
 =head1 TODO
 
+I will gladly accept help working on these:
+
 =over 2
 
 =item * Taproot compatibility
 
+=item * Better error checking (subroutine inputs, edge cases etc.)
+
+=item * Detailed manual
+
 =item * Better test coverage
 
 =back
@@ -212,7 +239,7 @@ Bartosz Jarzyna E<lt>bbrtj.pro@gmail.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2018 - 2022 by Bartosz Jarzyna
+Copyright (C) 2018 - 2023 by Bartosz Jarzyna
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
