@@ -1,14 +1,14 @@
 package Bitcoin::Crypto::BIP44;
-$Bitcoin::Crypto::BIP44::VERSION = '1.006_02'; # TRIAL
-$Bitcoin::Crypto::BIP44::VERSION = '1.00602';
+$Bitcoin::Crypto::BIP44::VERSION = '1.008_01'; # TRIAL
+$Bitcoin::Crypto::BIP44::VERSION = '1.00801';
 use v5.10;
 use strict;
 use warnings;
 use Moo;
-use Types::Standard qw(Enum);
-use Types::Common::Numeric qw(PositiveOrZeroInt);
 use Scalar::Util qw(blessed);
+use Mooish::AttributeBuilder -standard;
 
+use Bitcoin::Crypto::Types qw(BIP44Purpose PositiveOrZeroInt Bool Enum);
 use Bitcoin::Crypto::Network;
 use Bitcoin::Crypto::Exception;
 
@@ -19,7 +19,7 @@ sub _get_network_constant
 	my $coin_type = $network->bip44_coin;
 
 	Bitcoin::Crypto::Exception::NetworkConfig->raise(
-		"no bip44_coin constant found in network configuration"
+		'no bip44_coin constant found in network configuration'
 	) unless defined $coin_type;
 
 	return $coin_type;
@@ -27,14 +27,12 @@ sub _get_network_constant
 
 use namespace::clean;
 
-has 'purpose' => (
-	is => 'ro',
-	isa => Enum[44, 49, 84],
-	default => sub { 44 },
+has param 'purpose' => (
+	isa => BIP44Purpose,
+	default => 44,
 );
 
-has 'coin_type' => (
-	is => 'ro',
+has param 'coin_type' => (
 	isa => PositiveOrZeroInt,
 	coerce => sub {
 		my ($coin_type) = @_;
@@ -54,26 +52,33 @@ has 'coin_type' => (
 	},
 );
 
-has 'account' => (
-	is => 'ro',
+has param 'account' => (
 	isa => PositiveOrZeroInt,
-	default => sub { 0 },
+	default => 0,
 );
 
-has 'change' => (
-	is => 'ro',
+has param 'change' => (
 	isa => Enum [1, 0],
-	default => sub { 0 },
+	default => 0,
 );
 
-has 'index' => (
-	is => 'ro',
+has param 'index' => (
 	isa => PositiveOrZeroInt,
-	required => 1,
+	default => 0,
+);
+
+has param 'get_account' => (
+	isa => Bool,
+	default => !!0,
+);
+
+has param 'get_from_account' => (
+	isa => Bool,
+	default => !!0,
 );
 
 use overload
-	q{""} => "as_string",
+	q{""} => 'as_string',
 	fallback => 1;
 
 sub as_string
@@ -82,8 +87,18 @@ sub as_string
 
 	# https://github.com/bitcoin/bips/blob/master/bip-0044.mediawiki
 	# m / purpose' / coin_type' / account' / change / address_index
-	return sprintf "m/%u'/%u'/%u'/%u/%u",
-		$self->purpose, $self->coin_type, $self->account, $self->change, $self->index;
+
+	my $path = sprintf "m/%u'/%u'/%u'",
+		$self->purpose, $self->coin_type, $self->account;
+
+	return $path
+		if $self->get_account;
+
+	$path = 'm'
+		if $self->get_from_account;
+
+	return sprintf "%s/%u/%u",
+		$path, $self->change, $self->index;
 }
 
 1;
@@ -149,11 +164,26 @@ By default, the value C<0> is used.
 =head2 change
 
 Needs to be a number C<1> (for addresses to be used as change outputs) or C<0> (for addresses that are to be used only internally).
+
 By default, the value C<0> is used.
 
 =head2 index
 
-Required. Needs to be a non-negative integer number. It should be less than C<2^31> (but will not check for that).
+Needs to be a non-negative integer number. It should be less than C<2^31> (but will not check for that).
+
+By default, the value C<0> is used.
+
+=head2 get_account
+
+If passed C<1>, the resulting derivation path will only go as far as to the account part. L</index> and L</change> values will be ignored. Use this to get extended key for the account.
+
+By default, you will get the full derivation path.
+
+=head2 get_from_account
+
+If passed C<1>, the resulting derivation path will start after the account part. L</purpose>, L</coin_type> and L</account> values will be ignored. Use this to further derive key that was only derived up to the account part.
+
+By default, you will get the full derivation path.
 
 =head1 METHODS
 
