@@ -1,6 +1,6 @@
 package Bitcoin::Crypto::Base58;
-$Bitcoin::Crypto::Base58::VERSION = '1.006_02'; # TRIAL
-$Bitcoin::Crypto::Base58::VERSION = '1.00602';
+$Bitcoin::Crypto::Base58::VERSION = '1.008_01'; # TRIAL
+$Bitcoin::Crypto::Base58::VERSION = '1.00801';
 use v5.10;
 use strict;
 use warnings;
@@ -32,7 +32,7 @@ sub encode_base58
 sub encode_base58check
 {
 	my ($bytes) = @_;
-	my $checksum = pack("a" . $CHECKSUM_SIZE, hash256($bytes));
+	my $checksum = pack('a' . $CHECKSUM_SIZE, hash256($bytes));
 	return encode_base58($bytes . $checksum);
 }
 
@@ -41,7 +41,7 @@ sub decode_base58
 	my ($base58encoded) = @_;
 	my $decoded = decode_b58b($base58encoded);
 	Bitcoin::Crypto::Exception::Base58InputFormat->raise(
-		"illegal characters in base58 string"
+		'illegal characters in base58 string'
 	) unless defined $decoded;
 
 	return $decoded;
@@ -52,7 +52,7 @@ sub verify_checksum
 	my ($decoded) = @_;
 	my $encoded_val = substr $decoded, 0, -$CHECKSUM_SIZE;
 	my $checksum = substr $decoded, -$CHECKSUM_SIZE;
-	return unpack("a" . $CHECKSUM_SIZE, hash256($encoded_val)) eq $checksum;
+	return unpack('a' . $CHECKSUM_SIZE, hash256($encoded_val)) eq $checksum;
 }
 
 sub decode_base58check
@@ -61,7 +61,7 @@ sub decode_base58check
 	my $decoded = decode_base58($base58encoded);
 
 	Bitcoin::Crypto::Exception::Base58InputChecksum->raise(
-		"incorrect base58check checksum"
+		'incorrect base58check checksum'
 	) unless verify_checksum($decoded);
 
 	return substr $decoded, 0, -$CHECKSUM_SIZE;
@@ -84,7 +84,7 @@ Bitcoin::Crypto::Base58 - Bitcoin's Base58 helpers
 		decode_base58check
 	);
 
-	my $b58str = encode_base58check(pack "A*", "hello");
+	my $b58str = encode_base58check(pack 'A*', 'hello');
 	my $bytestr = decode_base58check($b58str);
 
 =head1 DESCRIPTION
