@@ -1,6 +1,6 @@
 package Bitcoin::Crypto::Helpers;
-$Bitcoin::Crypto::Helpers::VERSION = '1.006_02'; # TRIAL
-$Bitcoin::Crypto::Helpers::VERSION = '1.00602';
+$Bitcoin::Crypto::Helpers::VERSION = '1.008_01'; # TRIAL
+$Bitcoin::Crypto::Helpers::VERSION = '1.00801';
 use v5.10;
 use strict;
 use warnings;
@@ -13,8 +13,17 @@ use Crypt::PK::ECC;
 use Bitcoin::Crypto::Config;
 use Bitcoin::Crypto::Exception;
 
-# make sure Math::BigInt is loaded - this module loads it
-use Bitcoin::Crypto::Types;
+BEGIN {
+	require Math::BigInt;
+
+	# Version 1.6003 of optional GMP is required for the from_bytes / to_bytes implementations
+	if (eval { require Math::BigInt::GMP; Math::BigInt::GMP->VERSION('1.6003'); 1 }) {
+		Math::BigInt->import(try => 'GMP,LTM');
+	}
+	else {
+		Math::BigInt->import(try => 'LTM');
+	}
+}
 
 our @EXPORT_OK = qw(
 	new_bigint
@@ -29,14 +38,14 @@ our @EXPORT_OK = qw(
 sub new_bigint
 {
 	my ($bytes) = @_;
-	return Math::BigInt->from_hex(unpack "H*", $bytes);
+	return Math::BigInt->from_hex(unpack 'H*', $bytes);
 }
 
 sub pad_hex
 {
 	my ($hex) = @_;
 	$hex =~ s/^0x//;
-	return "0" x (length($hex) % 2) . $hex;
+	return '0' x (length($hex) % 2) . $hex;
 }
 
 sub ensure_length
@@ -56,13 +65,13 @@ sub verify_bytestring
 	my ($string) = @_;
 
 	Bitcoin::Crypto::Exception->raise(
-		"invalid input value, expected string"
+		'invalid input value, expected string'
 	) if !defined $string || ref $string;
 
 	my @characters = split //, $string;
 
 	Bitcoin::Crypto::Exception->raise(
-		"string contains characters with numeric values over 255 and cannot be used as a byte string"
+		'string contains characters with numeric values over 255 and cannot be used as a byte string'
 	) if (grep { ord($_) > 255 } @characters) > 0;
 }
 
@@ -91,8 +100,8 @@ sub add_ec_points
 
 	my $curve_size = Bitcoin::Crypto::Config::key_max_length;
 	my $curve_data = Crypt::PK::ECC->new->generate_key(Bitcoin::Crypto::Config::curve_name)->curve2hash;
-	my $p = new_bigint(pack "H*", $curve_data->{prime});
-	my $a = new_bigint(pack "H*", $curve_data->{A});
+	my $p = new_bigint(pack 'H*', $curve_data->{prime});
+	my $a = new_bigint(pack 'H*', $curve_data->{A});
 
 	my $add_points = sub {
 		my ($x1, $x2, $y1, $lambda) = @_;
@@ -154,3 +163,5 @@ sub add_ec_points
 
 1;
 
+# Internal use only
+
