@@ -1,26 +1,40 @@
 package Bitcoin::Crypto::Role::BasicKey;
-$Bitcoin::Crypto::Role::BasicKey::VERSION = '1.006_02'; # TRIAL
-$Bitcoin::Crypto::Role::BasicKey::VERSION = '1.00602';
+$Bitcoin::Crypto::Role::BasicKey::VERSION = '1.008_01'; # TRIAL
+$Bitcoin::Crypto::Role::BasicKey::VERSION = '1.00801';
 use v5.10;
 use strict;
 use warnings;
+use Carp qw(carp);
 
 use Bitcoin::Crypto::Helpers qw(pad_hex verify_bytestring);
 use Bitcoin::Crypto::Exception;
 use Moo::Role;
 
-with "Bitcoin::Crypto::Role::Key",
-	"Bitcoin::Crypto::Role::Compressed";
+with qw(
+	Bitcoin::Crypto::Role::Key
+	Bitcoin::Crypto::Role::Compressed
+);
+
+around BUILDARGS => sub {
+	my ($orig, $class, @params) = @_;
+
+	if (@params == 1) {
+		carp "$class->new(\$bytes) is now deprecated. Use $class->from_bytes(\$bytes) instead";
+		unshift @params, 'key_instance';
+	}
+
+	return $class->$orig(@params);
+};
 
 sub sign_message
 {
 	my ($self, $message, $algorithm) = @_;
 
 	Bitcoin::Crypto::Exception::Sign->raise(
-		"cannot sign a message with a public key"
+		'cannot sign a message with a public key'
 	) unless $self->_is_private;
 
-	$algorithm //= "sha256";
+	$algorithm //= 'sha256';
 	if (eval { require Crypt::Perl } && Crypt::Perl->VERSION gt '0.33') {
 		require Crypt::Perl::ECDSA::Parse;
 		$self->{_crypt_perl_prv} = Crypt::Perl::ECDSA::Parse::private($self->key_instance->export_key_der('private'))
@@ -50,7 +64,7 @@ sub verify_message
 	my ($self, $message, $signature, $algorithm) = @_;
 	verify_bytestring($signature);
 
-	$algorithm //= "sha256";
+	$algorithm //= 'sha256';
 	return Bitcoin::Crypto::Exception::Verify->trap_into(
 		sub {
 			$self->key_instance->verify_message($signature, $message, $algorithm);
@@ -61,13 +75,13 @@ sub verify_message
 sub from_hex
 {
 	my ($class, $val) = @_;
-	return $class->from_bytes(pack "H*", pad_hex($val));
+	return $class->from_bytes(pack 'H*', pad_hex($val));
 }
 
 sub to_hex
 {
 	my ($self) = @_;
-	return unpack "H*", $self->to_bytes();
+	return unpack 'H*', $self->to_bytes();
 }
 
 sub from_bytes
@@ -75,7 +89,7 @@ sub from_bytes
 	my ($class, $bytes) = @_;
 	verify_bytestring($bytes);
 
-	return $class->new($bytes);
+	return $class->new(key_instance => $bytes);
 }
 
 sub to_bytes
