@@ -1,20 +1,19 @@
 package Bitcoin::Crypto::Role::Compressed;
-$Bitcoin::Crypto::Role::Compressed::VERSION = '1.006_02'; # TRIAL
-$Bitcoin::Crypto::Role::Compressed::VERSION = '1.00602';
+$Bitcoin::Crypto::Role::Compressed::VERSION = '1.008_01'; # TRIAL
+$Bitcoin::Crypto::Role::Compressed::VERSION = '1.00801';
 use v5.10;
 use strict;
 use warnings;
-use Types::Standard qw(Bool);
+use Mooish::AttributeBuilder -standard;
 
+use Bitcoin::Crypto::Types qw(Bool);
 use Bitcoin::Crypto::Config;
 use Moo::Role;
 
-has "compressed" => (
-	is => "rw",
-	isa => Bool,
-	coerce => 1,
+has param 'compressed' => (
+	coerce => Bool,
 	default => Bitcoin::Crypto::Config::compress_public_point,
-	writer => "_set_compressed"
+	writer => -hidden,
 );
 
 sub set_compressed
