@@ -1,263 +1,55 @@
 package Bitcoin::Crypto::Script;
-$Bitcoin::Crypto::Script::VERSION = '1.006_02'; # TRIAL
-$Bitcoin::Crypto::Script::VERSION = '1.00602';
+$Bitcoin::Crypto::Script::VERSION = '1.008_01'; # TRIAL
+$Bitcoin::Crypto::Script::VERSION = '1.00801';
 use v5.10;
 use strict;
 use warnings;
 use Moo;
-use Types::Standard qw(ArrayRef Str);
 use Crypt::Digest::SHA256 qw(sha256);
+use Mooish::AttributeBuilder -standard;
 
 use Bitcoin::Crypto::Base58 qw(encode_base58check);
 use Bitcoin::Crypto::Bech32 qw(encode_segwit);
 use Bitcoin::Crypto::Config;
 use Bitcoin::Crypto::Helpers qw(hash160 hash256 verify_bytestring);
 use Bitcoin::Crypto::Exception;
+use Bitcoin::Crypto::Types qw(ArrayRef Str);
 
 use namespace::clean;
 
-with "Bitcoin::Crypto::Role::Network";
-
-# list of significant opcodes
-our %op_codes = (
-	FALSE => {
-		code => "\x00",
-	},
-	PUSHDATA1 => {
-		code => "\x4c",
-	},
-	PUSHDATA2 => {
-		code => "\x4d",
-	},
-	PUSHDATA4 => {
-		code => "\x4e",
-	},
-	"1NEGATE" => {
-		code => "\x4f",
-	},
-	RESERVED => {
-		code => "\x50",
-	},
-	TRUE => {
-		code => "\x51",
-	},
-	NOP => {
-		code => "\x61",
-	},
-	VER => {
-		code => "\x62",
-	},
-	IF => {
-		code => "\x63",
-	},
-	NOTIF => {
-		code => "\x64",
-	},
-	VERIF => {
-		code => "\x65",
-	},
-	VERNOTIF => {
-		code => "\x66",
-	},
-	ELSE => {
-		code => "\x67",
-	},
-	ENDIF => {
-		code => "\x68",
-	},
-	VERIFY => {
-		code => "\x69",
-	},
-	RETURN => {
-		code => "\x6a",
-	},
-	TOALTSTACK => {
-		code => "\x6b",
-	},
-	FROMALTSTACK => {
-		code => "\x6c",
-	},
-	"2DROP" => {
-		code => "\x6d",
-	},
-	"2DUP" => {
-		code => "\x6e",
-	},
-	"3DUP" => {
-		code => "\x6f",
-	},
-	"2OVER" => {
-		code => "\x70",
-	},
-	"2ROT" => {
-		code => "\x71",
-	},
-	"2SWAP" => {
-		code => "\x72",
-	},
-	IFDUP => {
-		code => "\x73",
-	},
-	DEPTH => {
-		code => "\x74",
-	},
-	DROP => {
-		code => "\x75",
-	},
-	DUP => {
-		code => "\x76",
-	},
-	NIP => {
-		code => "\x77",
-	},
-	OVER => {
-		code => "\x78",
-	},
-	PICK => {
-		code => "\x79",
-	},
-	ROLL => {
-		code => "\x7a",
-	},
-	ROT => {
-		code => "\x7b",
-	},
-	SWAP => {
-		code => "\x7c",
-	},
-	TUCK => {
-		code => "\x7d",
-	},
-	SIZE => {
-		code => "\x82",
-	},
-	EQUAL => {
-		code => "\x87",
-	},
-	EQUALVERIFY => {
-		code => "\x88",
-	},
-	RESERVED1 => {
-		code => "\x89",
-	},
-	RESERVED2 => {
-		code => "\x8a",
-	},
-	"1ADD" => {
-		code => "\x8b",
-	},
-	"1SUB" => {
-		code => "\x8c",
-	},
-	NEGATE => {
-		code => "\x8f",
-	},
-	ABS => {
-		code => "\x90",
-	},
-	NOT => {
-		code => "\x91",
-	},
-	ONOTEQUAL => {
-		code => "\x92",
-	},
-	ADD => {
-		code => "\x93",
-	},
-	SUB => {
-		code => "\x94",
-	},
-	BOOLAND => {
-		code => "\x9a",
-	},
-	BOOLOR => {
-		code => "\x9b",
-	},
-	NUMEQUAL => {
-		code => "\x9c",
-	},
-	NUMEQUALVERIFY => {
-		code => "\x9d",
-	},
-	NUMNOTEQUAL => {
-		code => "\x9e",
-	},
-	LESSTHAN => {
-		code => "\x9f",
-	},
-	GREATERTHAN => {
-		code => "\xa0",
-	},
-	LESSTHANOREQUAL => {
-		code => "\xa1",
-	},
-	GREATERTHANOREQUAL => {
-		code => "\xa2",
-	},
-	MIN => {
-		code => "\xa3",
-	},
-	MAX => {
-		code => "\xa4",
-	},
-	WITHIN => {
-		code => "\xa5",
-	},
-	RIPEMD160 => {
-		code => "\xa6",
-	},
-	SHA1 => {
-		code => "\xa7",
-	},
-	SHA256 => {
-		code => "\xa8",
-	},
-	HASH160 => {
-		code => "\xa9",
-	},
-	HASH256 => {
-		code => "\xaa",
-	},
-	CODESEPARATOR => {
-		code => "\xab",
-	},
-	CHECKSIG => {
-		code => "\xac",
-	},
-	CHECKSIGVERIFY => {
-		code => "\xad",
-	},
-	CHECKMULTISIG => {
-		code => "\xae",
-	},
-	CHECKMULTISIGVERIFY => {
-		code => "\xaf",
-	},
-	CHECKLOCKTIMEVERFIY => {
-		code => "\xb1",
-	},
-	CHECKSEQUENCEVERIFY => {
-		code => "\xb2",
-	},
-);
-
-$op_codes{0} = $op_codes{FALSE};
-$op_codes{1} = $op_codes{TRUE};
-
-for (2 .. 16) {
-
-	# OP_N - starts with 0x52, up to 0x60
-	$op_codes{$_} = {
-		code => pack("C", 0x50 + $_),
-	};
-}
-
-has "operations" => (
-	is => "rw",
+has field 'operations' => (
 	isa => ArrayRef [Str],
 	default => sub { [] },
 );
 
+with qw(Bitcoin::Crypto::Role::Network);
+
+# list of significant opcodes from DATA section
+our %op_codes = do {
+	my @list;
+	while (my $line = <DATA>) {
+		chomp $line;
+		last if $line eq '__END__';
+
+		my @parts = split /\s+/, $line;
+		next if @parts == 0;
+		die 'too many DATA parts for script opcode'
+			if @parts > 2;
+
+		# add key
+		push @list, shift @parts;
+
+		# rest of @parts are values
+		my ($code) = @parts;
+		push @list, {
+			code => pack('C', hex $code),
+		};
+	}
+
+	close DATA;
+	@list;
+};
+
 sub _get_op_code
 {
 	my ($context, $op_code) = @_;
@@ -268,11 +60,11 @@ sub _get_op_code
 	elsif ($op_code =~ /^[0-9]+$/ && $op_code >= 1 && $op_code <= 75) {
 
 		# standard data push - 0x01 up to 0x4b
-		return pack("C", 0x00 + $op_code);
+		return pack('C', 0x00 + $op_code);
 	}
 	else {
 		Bitcoin::Crypto::Exception::ScriptOpcode->raise(
-			defined $op_code ? "unknown opcode $op_code" : "undefined opcode variable"
+			defined $op_code ? "unknown opcode $op_code" : 'undefined opcode variable'
 		);
 	}
 }
@@ -301,11 +93,11 @@ sub push_bytes
 
 	my $len = length $bytes;
 	Bitcoin::Crypto::Exception::ScriptPush->raise(
-		"empty data variable"
+		'empty data variable'
 	) unless $len;
 
 	if ($bytes =~ /[\x00-\x10]/ && $len == 1) {
-		my $num = unpack "C", $bytes;
+		my $num = unpack 'C', $bytes;
 		$self->add_operation("OP_$num");
 	}
 	else {
@@ -314,20 +106,20 @@ sub push_bytes
 			$self->add_operation($len);
 		}
 		elsif ($len < (2 << 7)) {
-			$self->add_operation("OP_PUSHDATA1")
-				->add_raw(pack "C", $len);
+			$self->add_operation('OP_PUSHDATA1')
+				->add_raw(pack 'C', $len);
 		}
 		elsif ($len < (2 << 15)) {
-			$self->add_operation("OP_PUSHDATA2")
-				->add_raw(pack "S", $len);
+			$self->add_operation('OP_PUSHDATA2')
+				->add_raw(pack 'S', $len);
 		}
 		elsif ($len < (2 << 31)) {
-			$self->add_operation("OP_PUSHDATA4")
-				->add_raw(pack "L", $len);
+			$self->add_operation('OP_PUSHDATA4')
+				->add_raw(pack 'L', $len);
 		}
 		else {
 			Bitcoin::Crypto::Exception::ScriptPush->raise(
-				"too much data to push onto stack in one operation"
+				'too much data to push onto stack in one operation'
 			);
 		}
 		$self->add_raw($bytes);
@@ -338,7 +130,7 @@ sub push_bytes
 sub get_script
 {
 	my ($self) = @_;
-	return join "", @{$self->operations};
+	return join '', @{$self->operations};
 }
 
 sub get_script_hash
@@ -351,7 +143,7 @@ sub witness_program
 {
 	my ($self) = @_;
 
-	return pack("C", Bitcoin::Crypto::Config::witness_version) . sha256($self->get_script);
+	return pack('C', Bitcoin::Crypto::Config::witness_version) . sha256($self->get_script);
 }
 
 sub get_legacy_address
@@ -364,8 +156,13 @@ sub get_compat_address
 {
 	my ($self) = @_;
 
+	# network field is not required, lazy check for completeness
+	Bitcoin::Crypto::Exception::NetworkConfig->raise(
+		'this network does not support segregated witness'
+	) unless $self->network->supports_segwit;
+
 	my $program = Bitcoin::Crypto::Script->new(network => $self->network);
-	$program->add_operation("OP_" . Bitcoin::Crypto::Config::witness_version)
+	$program->add_operation('OP_' . Bitcoin::Crypto::Config::witness_version)
 		->push_bytes(sha256($self->get_script));
 	return $program->get_legacy_address;
 }
@@ -376,14 +173,107 @@ sub get_segwit_address
 
 	# network field is not required, lazy check for completeness
 	Bitcoin::Crypto::Exception::NetworkConfig->raise(
-		"no segwit_hrp found in network configuration"
-	) unless defined $self->network->segwit_hrp;
+		'this network does not support segregated witness'
+	) unless $self->network->supports_segwit;
 
 	return encode_segwit($self->network->segwit_hrp, $self->witness_program);
 }
 
 1;
 
+__DATA__
+
+0                    00
+FALSE                00
+PUSHDATA1            4c
+PUSHDATA2            4d
+PUSHDATA4            4e
+1NEGATE              4f
+RESERVED             50
+TRUE                 51
+1                    51
+2                    52
+3                    53
+4                    54
+5                    55
+6                    56
+7                    57
+8                    58
+9                    59
+10                   5a
+11                   5b
+12                   5c
+13                   5d
+14                   5e
+15                   5f
+16                   60
+NOP                  61
+VER                  62
+IF                   63
+NOTIF                64
+VERIF                65
+VERNOTIF             66
+ELSE                 67
+ENDIF                68
+VERIFY               69
+RETURN               6a
+TOALTSTACK           6b
+FROMALTSTACK         6c
+2DROP                6d
+2DUP                 6e
+3DUP                 6f
+2OVER                70
+2ROT                 71
+2SWAP                72
+IFDUP                73
+DEPTH                74
+DROP                 75
+DUP                  76
+NIP                  77
+OVER                 78
+PICK                 79
+ROLL                 7a
+ROT                  7b
+SWAP                 7c
+TUCK                 7d
+SIZE                 82
+EQUAL                87
+EQUALVERIFY          88
+RESERVED1            89
+RESERVED2            8a
+1ADD                 8b
+1SUB                 8c
+NEGATE               8f
+ABS                  90
+NOT                  91
+ONOTEQUAL            92
+ADD                  93
+SUB                  94
+BOOLAND              9a
+BOOLOR               9b
+NUMEQUAL             9c
+NUMEQUALVERIFY       9d
+NUMNOTEQUAL          9e
+LESSTHAN             9f
+GREATERTHAN          a0
+LESSTHANOREQUAL      a1
+GREATERTHANOREQUAL   a2
+MIN                  a3
+MAX                  a4
+WITHIN               a5
+RIPEMD160            a6
+SHA1                 a7
+SHA256               a8
+HASH160              a9
+HASH256              aa
+CODESEPARATOR        ab
+CHECKSIG             ac
+CHECKSIGVERIFY       ad
+CHECKMULTISIG        ae
+CHECKMULTISIGVERIFY  af
+CHECKLOCKTIMEVERFIY  b1
+CHECKSEQUENCEVERIFY  b2
+
 __END__
 =head1 NAME
 
@@ -394,9 +284,9 @@ Bitcoin::Crypto::Script - Bitcoin script representations
 	use Bitcoin::Crypto::Script;
 
 	my $script = Bitcoin::Crypto::Script->new
-		->add_operation("OP_1")
-		->add_operation("OP_TRUE")
-		->add_operation("OP_EQUAL");
+		->add_operation('OP_1')
+		->add_operation('OP_TRUE')
+		->add_operation('OP_EQUAL');
 
 	# getting serialized script
 	my $serialized = $script->get_script();
@@ -424,9 +314,9 @@ You can use a script object to:
 
 =head2 new
 
-	$script_object = $class->new($data)
+	$script_object = $class->new()
 
-A constructor. Returns new script instance
+A constructor. Returns new script instance.
 
 =head2 add_operation
 
@@ -451,7 +341,7 @@ Returns the object instance for chaining.
 
 Pushes C<$bytes> to the execution stack at the end of a script, using a minimal push opcode.
 
-For example, running C<$script->push_bytes("\x03")> will have the same effect as C<$script->add_operation("OP_3")>.
+For example, running C<$script->push_bytes("\x03")> will have the same effect as C<$script->add_operation('OP_3')>.
 
 Throws an exception for data exceeding a 4 byte number in length.
 
@@ -513,7 +403,7 @@ This module throws an instance of L<Bitcoin::Crypto::Exception> if it encounters
 
 =over 2
 
-=item L<Bitcoin::Crypto::PrivateKey>
+=item L<Bitcoin::Crypto::Key::Private>
 
 =item L<Bitcoin::Crypto::Network>
 
