@@ -1,6 +1,6 @@
 package Bitcoin::Crypto::Segwit;
-$Bitcoin::Crypto::Segwit::VERSION = '1.006_02'; # TRIAL
-$Bitcoin::Crypto::Segwit::VERSION = '1.00602';
+$Bitcoin::Crypto::Segwit::VERSION = '1.008_01'; # TRIAL
+$Bitcoin::Crypto::Segwit::VERSION = '1.00801';
 use v5.10;
 use strict;
 use warnings;
@@ -21,7 +21,7 @@ our %validators = (
 		my ($data) = @_;
 
 		Bitcoin::Crypto::Exception::SegwitProgram->raise(
-			"incorrect witness program length"
+			'incorrect witness program length'
 		) unless length $data == 20 || length $data == 32;
 		return;
 	},
@@ -32,7 +32,7 @@ sub common_validator
 	my ($data) = @_;
 
 	Bitcoin::Crypto::Exception::SegwitProgram->raise(
-		"incorrect witness program length"
+		'incorrect witness program length'
 	) unless length $data >= 2 && length $data <= 40;
 	return;
 }
@@ -42,9 +42,9 @@ sub validate_program
 	my ($program) = @_;
 	verify_bytestring($program);
 
-	my $version = unpack "C", $program;
+	my $version = unpack 'C', $program;
 	Bitcoin::Crypto::Exception::SegwitProgram->raise(
-		"incorrect witness program version " . ($version // "[null]")
+		'incorrect witness program version ' . ($version // '[null]')
 	) unless defined $version && $version >= 0 && $version <= Bitcoin::Crypto::Config::max_witness_version;
 
 	$program = substr $program, 1;
@@ -97,7 +97,7 @@ The current implementation defines a validator for segwit version 0. In the futu
 
 		# perform validation
 		Bitcoin::Crypto::Exception::SegwitProgram->raise(
-			"validation of program version 1 failed"
+			'validation of program version 1 failed'
 		) if ...;
 
 		# if validation is successful just do nothing
